@@ -21,11 +21,15 @@ interGeneCorrelation <- function(y, design)
 
 camera <- function(y,...) UseMethod("camera")
 
-camera.default <- function(y,index,design=NULL,contrast=ncol(design),weights=NULL,use.ranks=FALSE,allow.neg.cor=TRUE,trend.var=FALSE,sort=TRUE,...)
+camera.default <- function(y,index,design=NULL,contrast=ncol(design),weights=NULL,use.ranks=FALSE,allow.neg.cor=TRUE,inter.gene.cor=NULL,trend.var=FALSE,sort=TRUE,...)
 #	Competitive gene set test allowing for correlation between genes
 #	Gordon Smyth and Di Wu
-#	Created 2007.  Last modified 25 Feb 2014
+#	Created 2007.  Last modified 20 July 2015
 {
+#	Issue warning if extra arguments found
+	dots <- names(list(...))
+	if(length(dots)) warning("Extra arguments disregarded: ",sQuote(dots))
+
 #	Extract components from y
 	y <- getEAWP(y)
 	G <- nrow(y$exprs)
@@ -33,6 +37,8 @@ camera.default <- function(y,index,design=NULL,contrast=ncol(design),weights=NUL
 
 #	Check index
 	if(!is.list(index)) index <- list(set1=index)
+	nsets <- length(index)
+	if(nsets==0) stop("index is empty")
 
 #	Check design
 	if(is.null(design)) design <- y$design
@@ -45,14 +51,24 @@ camera.default <- function(y,index,design=NULL,contrast=ncol(design),weights=NUL
 	if(nrow(design) != n) stop("row dimension of design matrix must match column dimension of data")
 	ne <- nonEstimable(design)
 	if(!is.null(ne)) cat("Coefficients not estimable:",paste(ne,collapse=" "),"\n")
-
 	p <- ncol(design)
 	df.residual <- n-p
-	df.camera <- min(df.residual,G-2)
 
 #	Check weights
 	if(is.null(weights)) weights <- y$weights
 
+#	Check inter.gene.cor
+	fixed.cor <- !is.null(inter.gene.cor)
+
+#	Set for camera tests
+	if(fixed.cor)
+		if(use.ranks)
+			df.camera <- Inf
+		else
+			df.camera <- G-2
+	else
+		df.camera <- min(df.residual,G-2)
+
 #	Reduce to numeric expression matrix
 	y <- y$exprs
 
@@ -112,20 +128,23 @@ camera.default <- function(y,index,design=NULL,contrast=ncol(design),weights=NUL
 #	Standardized residuals
 	U <- effects[-(1:p),,drop=FALSE]
 	sigma2 <- colMeans(U^2)
-	U <- t(U) / sqrt(sigma2)
+	U <- t(U) / sqrt(pmax(sigma2,1e-8))
 
 #	Moderated t
 	if(trend.var) A <- rowMeans(y) else A <- NULL
 	sv <- squeezeVar(sigma2,df=df.residual,covariate=A)
 	modt <- unscaledt / sqrt(sv$var.post)
-	df.total <- min(df.residual+sv$df.prior, G*df.residual)
-	Stat <- zscoreT(modt, df=df.total)
+	if(use.ranks)
+		Stat <- modt
+	else {
+		df.total <- min(df.residual+sv$df.prior, G*df.residual)
+		Stat <- zscoreT(modt, df=df.total, approx=TRUE)
+	}
 
 #	Global statistics
 	meanStat <- mean(Stat)
 	varStat <- var(Stat)
 
-	nsets <- length(index)
 	tab <- matrix(0,nsets,5)
 	rownames(tab) <- names(index)
 	colnames(tab) <- c("NGenes","Correlation","Down","Up","TwoSided")
@@ -134,13 +153,18 @@ camera.default <- function(y,index,design=NULL,contrast=ncol(design),weights=NUL
 		StatInSet <- Stat[iset]
 		m <- length(StatInSet)
 		m2 <- G-m
-		if(m>1) {
-			Uset <- U[iset,,drop=FALSE]
-			vif <- m * mean(colMeans(Uset)^2)
-			correlation <- (vif-1)/(m-1)
+		if(fixed.cor) {
+			correlation <- inter.gene.cor
+			vif <- 1+(m-1)*correlation
 		} else {
-			vif <- 1
-			correlation <- NA
+			if(m>1) {
+				Uset <- U[iset,,drop=FALSE]
+				vif <- m * mean(colMeans(Uset)^2)
+				correlation <- (vif-1)/(m-1)
+			} else {
+				vif <- 1
+				correlation <- NA
+			}
 		}
 
 		tab[i,1] <- m
