@@ -16,8 +16,12 @@ roast <- function(y,...) UseMethod("roast")
 roast.default <- function(y,index=NULL,design=NULL,contrast=ncol(design),set.statistic="mean",gene.weights=NULL,array.weights=NULL,weights=NULL,block=NULL,correlation,var.prior=NULL,df.prior=NULL,trend.var=FALSE,nrot=999,approx.zscore=TRUE,...)
 # Rotation gene set testing for linear models
 # Gordon Smyth and Di Wu
-# Created 24 Apr 2008.  Last modified 19 Sep 2014.
+# Created 24 Apr 2008.  Last modified 16 July 2015.
 {
+#	Issue warning if extra arguments found
+	dots <- names(list(...))
+	if(length(dots)) warning("Extra arguments disregarded: ",sQuote(dots))
+
 #	Check index
 	if(is.list(index)) return(mroast(y=y,index=index,design=design,contrast=contrast,set.statistic=set.statistic,gene.weights=gene.weights,array.weights=array.weights,weights=weights,block=block,correlation=correlation,var.prior=var.prior,df.prior=df.prior,trend.var=trend.var,nrot=nrot))
 
@@ -93,7 +97,9 @@ roast.default <- function(y,index=NULL,design=NULL,contrast=ncol(design),set.sta
 		diag(cormatrix) <- 1
 		R <- chol(cormatrix)
 		y <- t(backsolve(R, t(y), transpose = TRUE))
+		dn <- dimnames(design)
 		design <- backsolve(R, design, transpose = TRUE)
+		dimnames(design) <- dn
  	}
 
 #	Check contrast
@@ -351,7 +357,7 @@ mroast <- function(y,...) UseMethod("mroast")
 mroast.default <- function(y,index=NULL,design=NULL,contrast=ncol(design),set.statistic="mean",gene.weights=NULL,array.weights=NULL,weights=NULL,block=NULL,correlation,var.prior=NULL,df.prior=NULL,trend.var=FALSE,nrot=999,approx.zscore=TRUE,adjust.method="BH",midp=TRUE,sort="directional",...)
 #  Rotation gene set testing with multiple sets
 #  Gordon Smyth and Di Wu
-#  Created 28 Jan 2010. Last revised 18 Feb 2014.
+#  Created 28 Jan 2010. Last revised 23 June 2015.
 {
 #	Extract components from y
 	y <- getEAWP(y)
@@ -362,6 +368,7 @@ mroast.default <- function(y,index=NULL,design=NULL,contrast=ncol(design),set.st
 	if(is.null(index)) index <- rep(TRUE,ngenes)
 	if(!is.list(index)) index <- list(set = index)
 	nsets <- length(index)
+	if(nsets==0) stop("index is empty")
 	if(is.null(names(index))) names(index) <- paste("set",1:nsets,sep="")
 
 #	Check design matrix
@@ -479,6 +486,11 @@ mroast.default <- function(y,index=NULL,design=NULL,contrast=ncol(design),set.st
 		stringsAsFactors=FALSE
 	)
 
+	if(midp) {
+		tab$FDR <- pmax(tab$FDR, pv[,"UpOrDown"])
+		tab$FDR.Mixed <- pmax(tab$FDR.Mixed, pv[,"Mixed"])
+	}
+
 #	Sort by p-value
 	sort <- match.arg(sort,c("directional","mixed","none"))
 	if(sort=="none") return(tab)
