@@ -3,7 +3,7 @@
 lmFit <- function(object,design=NULL,ndups=1,spacing=1,block=NULL,correlation,weights=NULL,method="ls",...)
 #	Fit genewise linear models
 #	Gordon Smyth
-#	30 June 2003.  Last modified 28 July 2013.
+#	30 June 2003.  Last modified 7 July 2015.
 {
 #	Extract components from y
 	y <- getEAWP(object)
@@ -15,6 +15,7 @@ lmFit <- function(object,design=NULL,ndups=1,spacing=1,block=NULL,correlation,we
 	else {
 		design <- as.matrix(design)
 		if(mode(design) != "numeric") stop("design must be a numeric matrix")
+		if(nrow(design) != ncol(y$exprs)) stop("row dimension of design doesn't match column dimension of data object")
 	}
 	ne <- nonEstimable(design)
 	if(!is.null(ne)) cat("Coefficients not estimable:",paste(ne,collapse=" "),"\n")
@@ -67,7 +68,7 @@ lmFit <- function(object,design=NULL,ndups=1,spacing=1,block=NULL,correlation,we
 lm.series <- function(M,design=NULL,ndups=1,spacing=1,weights=NULL)
 #	Fit linear model for each gene to a series of arrays
 #	Gordon Smyth
-#	18 Apr 2002. Revised 16 Apr 2013.
+#	18 Apr 2002. Revised 26 June 2015.
 {
 #	Check expression matrix
 	M <- as.matrix(M)
@@ -102,7 +103,7 @@ lm.series <- function(M,design=NULL,ndups=1,spacing=1,weights=NULL)
 
 #	Check whether QR-decomposition is constant for all genes
 #	If so, fit all genes in one sweep
-	NoProbeWts <- !any(is.na(M)) && (is.null(weights) || !is.null(attr(weights,"arrayweights")))
+	NoProbeWts <- all(is.finite(M)) && (is.null(weights) || !is.null(attr(weights,"arrayweights")))
 	if(NoProbeWts) {
 		if(is.null(weights))
 			fit <- lm.fit(design, t(M))
@@ -130,7 +131,7 @@ lm.series <- function(M,design=NULL,ndups=1,spacing=1,weights=NULL)
 		return(fit)
 	}
 
-#	Genewise QR-decompositions are required, so iteration through genes
+#	Genewise QR-decompositions are required, so iterate through genes
 	beta <- stdev.unscaled
 	sigma <- rep(NA,ngenes)
 	df.residual <- rep(0,ngenes)
@@ -160,15 +161,15 @@ lm.series <- function(M,design=NULL,ndups=1,spacing=1,weights=NULL)
 	est <- QR$pivot[1:QR$rank]
 	dimnames(cov.coef) <- list(coef.names[est],coef.names[est])
 
-	list(coefficients=beta,stdev.unscaled=stdev.unscaled,sigma=sigma,df.residual=df.residual,cov.coefficients=cov.coef,pivot=QR$pivot)
+	list(coefficients=beta,stdev.unscaled=stdev.unscaled,sigma=sigma,df.residual=df.residual,cov.coefficients=cov.coef,pivot=QR$pivot,rank=QR$rank)
 }
 
 mrlm <- function(M,design=NULL,ndups=1,spacing=1,weights=NULL,...)
 #	Robustly fit linear model for each gene to a series of arrays
 #	Gordon Smyth
-#	20 Mar 2002.  Last revised 22 Feb 2007.
+#	20 Mar 2002.  Last revised 26 June 2015.
 {
-	require(MASS) # need rlm.default
+	if(!requireNamespace("MASS",quietly=TRUE)) stop("MASS package required but is not available")
 	M <- as.matrix(M)
 	narrays <- ncol(M)
 	if(is.null(design)) design <- matrix(1,narrays,1)
@@ -199,7 +200,7 @@ mrlm <- function(M,design=NULL,ndups=1,spacing=1,weights=NULL,...)
 		else
 			w <- as.vector(weights[i,obs])
 		if(length(y) > nbeta) {
-			out <- rlm(x=X,y=y,weights=w,...)
+			out <- MASS::rlm(x=X,y=y,weights=w,...)
 			beta[i,] <- coef(out)
 			stdev.unscaled[i,] <- sqrt(diag(chol2inv(out$qr$qr)))
 			df.residual[i] <- length(y) - out$rank
@@ -210,14 +211,14 @@ mrlm <- function(M,design=NULL,ndups=1,spacing=1,weights=NULL,...)
 	cov.coef <- chol2inv(QR$qr,size=QR$rank)
 	est <- QR$pivot[1:QR$rank]
 	dimnames(cov.coef) <- list(coef.names[est],coef.names[est])
-	list(coefficients=beta,stdev.unscaled=stdev.unscaled,sigma=sigma,df.residual=df.residual,cov.coefficients=cov.coef,pivot=QR$pivot)
+	list(coefficients=beta,stdev.unscaled=stdev.unscaled,sigma=sigma,df.residual=df.residual,cov.coefficients=cov.coef,pivot=QR$pivot,rank=QR$rank)
 }
 
 gls.series <- function(M,design=NULL,ndups=2,spacing=1,block=NULL,correlation=NULL,weights=NULL,...)
 #	Fit linear model for each gene to a series of microarrays.
 #	Fit is by generalized least squares allowing for correlation between duplicate spots.
 #	Gordon Smyth
-#	11 May 2002.  Last revised 12 Dec 2010.
+#	11 May 2002.  Last revised 26 June 2015.
 {
 	M <- as.matrix(M)
 	narrays <- ncol(M)
@@ -262,7 +263,7 @@ gls.series <- function(M,design=NULL,ndups=2,spacing=1,block=NULL,correlation=NU
 	ngenes <- nrow(M)
 	stdev.unscaled <- matrix(NA,ngenes,nbeta,dimnames=list(rownames(M),coef.names))
 
-	NoProbeWts <- !any(is.na(M)) && (is.null(weights) || !is.null(attr(weights,"arrayweights")))
+	NoProbeWts <- all(is.finite(M)) && (is.null(weights) || !is.null(attr(weights,"arrayweights")))
 	if(NoProbeWts) {
 		V <- cormatrix
 		if(!is.null(weights)) {
@@ -336,7 +337,7 @@ gls.series <- function(M,design=NULL,ndups=2,spacing=1,block=NULL,correlation=NU
 	cov.coef <- chol2inv(QR$qr,size=QR$rank)
 	est <- QR$pivot[1:QR$rank]
 	dimnames(cov.coef) <- list(coef.names[est],coef.names[est])
-	list(coefficients=beta,stdev.unscaled=stdev.unscaled,sigma=sigma,df.residual=df.residual,ndups=ndups,spacing=spacing,block=block,correlation=correlation,cov.coefficients=cov.coef,pivot=QR$pivot)
+	list(coefficients=beta,stdev.unscaled=stdev.unscaled,sigma=sigma,df.residual=df.residual,ndups=ndups,spacing=spacing,block=block,correlation=correlation,cov.coefficients=cov.coef,pivot=QR$pivot,rank=QR$rank)
 }
 
 is.fullrank <- function(x)
@@ -390,7 +391,7 @@ getEAWP <- function(object)
 #	Given any microarray data object, extract basic information needed
 #	for linear modelling.
 #	Gordon Smyth
-#	9 March 2008. Last modified 19 Sep 2014.
+#	9 March 2008. Last modified 7 July 2015.
 {
 	y <- list()
 	
@@ -409,10 +410,11 @@ getEAWP <- function(object)
 		y$design <- object$design
 	} else {
 	if(is(object,"ExpressionSet")) {
-		y$exprs <- exprs(object)
+		if(!requireNamespace("Biobase",quietly=TRUE)) stop("Biobase package required but is not available")
+		y$exprs <- Biobase::exprs(object)
 		if(length(object@featureData@data)) y$probes <- object@featureData@data
 		y$Amean <- rowMeans(y$exprs,na.rm=TRUE)
-		if("weights" %in% assayDataElementNames(object)) y$weights <- assayDataElement(object,"weights")
+		if("weights" %in% Biobase::assayDataElementNames(object)) y$weights <- Biobase::assayDataElement(object,"weights")
 	} else {
 	if(is(object,"PLMset")) {
 		y$exprs <- object@chip.coefs
@@ -430,9 +432,10 @@ getEAWP <- function(object)
 		if(length(object@maA)) y$Amean <- rowMeans(object@maA,na.rm=TRUE)
 	} else {
 #		Default method for matrices, data.frames, vsn objects etc.
-		y$exprs <- as.matrix(object)
-#		If exprs are positive, assume they are log-intensities rather than log-ratios
-#		if(all(y$exprs>=0,na.rm=TRUE)) y$Amean <- rowMeans(y$exprs,na.rm=TRUE)
+		if(is.vector(object))
+			y$exprs <- matrix(object,nrow=1)
+		else
+			y$exprs <- as.matrix(object)
 		y$Amean <- rowMeans(y$exprs,na.rm=TRUE)
 	}}}}
 
